import { trace } from '@opentelemetry/api';
import { logs, SeverityNumber } from '@opentelemetry/api-logs';
import { getNodeAutoInstrumentations } from '@opentelemetry/auto-instrumentations-node';
import { OTLPLogExporter } from '@opentelemetry/exporter-logs-otlp-http';
import { OTLPMetricExporter } from '@opentelemetry/exporter-metrics-otlp-http';
import { OTLPTraceExporter } from '@opentelemetry/exporter-trace-otlp-http';
import { Resource } from '@opentelemetry/resources';
import { BatchLogRecordProcessor } from '@opentelemetry/sdk-logs';
import { PeriodicExportingMetricReader } from '@opentelemetry/sdk-metrics';
import { NodeSDK } from '@opentelemetry/sdk-node';
import { SemanticResourceAttributes } from '@opentelemetry/semantic-conventions';
import dotenv from 'dotenv';
import express, { Response } from 'express';
dotenv.config();

// For troubleshooting, set the log level to DiagLogLevel.DEBUG
// diag.setLogger(new DiagConsoleLogger(), DiagLogLevel.INFO);
//

const env = {
	service: {
		name: process.env.SERVER_HOSTNAME || 'undefined service',
		version: process.env.npm_package_version || 'undefined service version',
	},
	collector: {
		url: process.env.OPENTELEMETRY_COLLECTOR_URL || 'http://otelcol:4318',
	},
};

const sdk = new NodeSDK({
	resource: new Resource({
		[SemanticResourceAttributes.SERVICE_NAME]: env.service.name,
		[SemanticResourceAttributes.SERVICE_VERSION]: env.service.version,
	}),
	traceExporter: new OTLPTraceExporter({
		url: `${env.collector.url}/v1/traces`,
	}),
	metricReader: new PeriodicExportingMetricReader({
		exporter: new OTLPMetricExporter({
			url: `${env.collector.url}/v1/metrics`,
		}),
	}),
	logRecordProcessor: new BatchLogRecordProcessor(
		new OTLPLogExporter({
			url: `${env.collector.url}/v1/logs`,
		})
	),
	instrumentations: [getNodeAutoInstrumentations()],
});

const app = express();
/**
 * Open Telemtry Logging middleware
 */

const loggingMiddleware = app.use((req, res, next) => {
	console.log('IM HERE');
	console.log('ENV', env);
	try {
		const activeSpan = trace.getActiveSpan();
		const logger = logs.getLogger(env.service.name);

		console.log('activeSpan', activeSpan);
		console.log('logger', logger);

		if (activeSpan !== undefined && logger !== undefined) {
			console.log('IN!');
			const temp = res.json;

			res.json = function (...args): Response<unknown, Record<string, unknown>> {
				logger.emit({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					body: JSON.stringify({ context: 'request headers', message: req.headers }),
				});

				logger.emit({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					body: JSON.stringify({ context: 'request body', message: req.body }),
				});

				logger.emit({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					body: JSON.stringify({ context: 'request query', message: req.query }),
				});

				logger.emit({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					body: JSON.stringify({ context: 'request params', message: req.params }),
				});

				logger.emit({
					severityNumber: SeverityNumber.INFO,
					severityText: 'INFO',
					body: JSON.stringify({ context: 'response', message: args['0'] }),
				});

				return temp.apply(this, args);
			};
		}
	} catch (e) {
		console.log('--- ERR :( ---', e);
	}
	next();
});

const logger = logs.getLogger(env.service.name);

export { logger, loggingMiddleware, sdk, trace };
